int gridSize = 32;
int domainWidth = 30;
int domainHeight = 20;
World world;

color wallColor = color(50);
color outerWallColor = color(0);
color startColor = color(255, 0, 0);
color goalColor = color(0, 0, 255);

EditMode editMode = EditMode.PUT_WALL;

void setup() {
  //size(gridSize * domainWidth, gridSize * domainHeight,"processing.core.PGraphicsRetina2D");
  size(gridSize * domainWidth, gridSize * domainHeight);
  world = new World(domainWidth, domainHeight);
  background(0);
}
void draw() {
  background(0);
  world.update();

  drawTerrain();
  drawMouseCursor();
  drawInfo();
}

void paintTerrain() {
  //Clip mouse position if over window
  int x = mouseX;
  int y = mouseY;
  if (x < 0) x = 0;
  if (x >= width) x = width-1;
  if (y < 0) y = 0;
  if (y >= height) y = height-1;

  if (editMode == EditMode.PUT_WALL) {
    world.setTerrainType(x/gridSize, y/gridSize,NodeType.WALL);
  } else if (editMode == EditMode.REMOVE_WALL) {
    world.setTerrainType(x/gridSize, y/gridSize,NodeType.NONE);
  } else if (editMode == EditMode.PUT_START) {
    world.setTerrainType(x/gridSize, y/gridSize,NodeType.START);
  } else if (editMode == EditMode.PUT_GOAL) {
    world.setTerrainType(x/gridSize, y/gridSize,NodeType.GOAL);
  }
}

void drawTerrain() {
  colorMode(RGB);
  stroke(128);
  strokeWeight(1);
  for (int j = 0; j < domainHeight; j++) {
    for (int i = 0; i < domainWidth; i++) {
      Node n = world.getNode(i, j);
      if (n.nodeType == NodeType.WALL) {
        fill(wallColor);
      } else if (n.nodeType == NodeType.OUTER) {
        fill(outerWallColor);
      } else if (n.nodeType == NodeType.START) {
        fill(startColor);
      } else if (n.nodeType == NodeType.GOAL) {
        fill(goalColor);
      } else {
        fill(255);
      }
      if (n.isRoute && n.nodeType != NodeType.START && n.nodeType != NodeType.GOAL) {
        fill(128);
      }

      rect(i*gridSize, j*gridSize, i*gridSize + gridSize, j*gridSize + gridSize);

      fill(0, 0, 0);
      textSize(8);
      //draw score
      if (n.nodeType == NodeType.NONE && n.score != Integer.MAX_VALUE) {
        text("s=" + n.score, i*gridSize+3, j*gridSize+8);
        text("f=" + n.fs, i*gridSize+3, j*gridSize+16);
      }
      text(n.done?"true":"false", i*gridSize+3, j*gridSize+24);
    }
  }
}

void mousePressed() {
  paintTerrain();
}

void mouseDragged() {
  paintTerrain();
}


void keyReleased() {
  switch(key) {
  case 'w':
    editMode = EditMode.PUT_WALL;
    break;
  case 'e':
    editMode = EditMode.REMOVE_WALL;
    break;
  case 's':
    editMode = EditMode.PUT_START;
    break;
  case 'g':
    editMode = EditMode.PUT_GOAL;
    break;
  }
}

void drawMouseCursor() {
  //DrawMouseOverRect
  stroke(0);
  noFill();
  strokeWeight(3);
  rect(mouseX/gridSize * gridSize, mouseY/gridSize * gridSize, gridSize, gridSize);
}

void drawInfo() {
  String mode = "";
  switch(editMode) {
  case PUT_WALL:
    mode = "Put wall";
    break;
  case REMOVE_WALL:
    mode = "Remove wall";
    break;
  case PUT_START:
    mode = "Put start point";
    break;
  case PUT_GOAL:
    mode = "Put gaol point";
    break;
  }
  fill(255);
  textSize(18);
  text("Mode:" + mode, 10, 20);
  textSize(18);
  fill(255);
  text("w key:put wall / e key:remove wall / s key:put start / g key:put goal", 10, height-5);
}
