class World {
  private ArrayList<Node> nodes;
  private ArrayList<Node> openNodes;
  private int worldWidth;
  private int worldHeight;
  public Node startNode;
  public Node goalNode;
  
  private int[] dx = {
    0, 1, 0, -1
  };
  private int[] dy = {
    1, 0, -1, 0
  };
  
  private boolean isWorldEdited = true;
  
  World(int _worldWidth, int _worldHeight) {
    this.worldWidth = _worldWidth;
    this.worldHeight = _worldHeight;

    startNode = null;
    goalNode = null;
    nodes = new ArrayList<Node>();
    openNodes = new ArrayList<Node>();
    for (int j = 0; j < worldHeight; j++) {
      for (int i = 0; i < worldWidth; i++) {
        nodes.add(new Node((j*worldWidth)+i, i, j));
      }
    }

    //init wall;
    for (int i = 0; i < worldWidth; i++) {
      getNode(i, 0).nodeType = NodeType.OUTER;
      getNode(i, worldHeight-1).nodeType = NodeType.OUTER;
    }
    for (int j = 0; j < worldHeight; j++) {
      getNode(0, j).nodeType = NodeType.OUTER;
      getNode(worldWidth-1, j).nodeType = NodeType.OUTER;
    }
    //init start and goal point
    getNode(1, 1).nodeType = NodeType.START;
    startNode = getNode(1, 1);
    getNode(worldWidth-2, worldHeight-2).nodeType = NodeType.GOAL;
    goalNode = getNode(worldWidth-2, worldHeight-2);
  }

  void update() {
    if(isWorldEdited){
    resetNodes();
    world.solveStart();
    isWorldEdited = false;
    //Detect route node
    for (int j = 0; j < domainHeight; j++) {
      for (int i = 0; i < domainWidth; i++) {
        Node n = getNode(i, j);
        if (n.nodeType == NodeType.GOAL && n.done) {
          n.isRoute = true;
          Node p = n.prev;
          while (p != null) {
            p.isRoute = true;
            p = p.prev;
          }
        }
      }
    }
    }
  }


  Node getNode(int x, int y) {
    return nodes.get((y*worldWidth)+x);
  }

  void resetNodes() {
    openNodes.clear();
    for (int j = 0; j < worldHeight; j++) {
      for (int i = 0; i < worldWidth; i++) {
        getNode(i, j).reset();
      }
    }
  }

  void setTerrainType(int x, int y, NodeType type) {
    Node n = getNode(x,y);
    if (type == NodeType.WALL) {
      if (n.nodeType == NodeType.NONE)n.nodeType = NodeType.WALL;
    } else if (type == NodeType.NONE) {
      if (n.nodeType == NodeType.WALL)n.nodeType = NodeType.NONE;
    } else if (type == NodeType.START) {
      if (n.nodeType == NodeType.NONE) {
        if (startNode != null)startNode.nodeType = NodeType.NONE;
        startNode = n;
        n.nodeType = NodeType.START;
      }
    } else if (type == NodeType.GOAL) {
      if (n.nodeType == NodeType.NONE) {
        if (goalNode != null)goalNode.nodeType = NodeType.NONE;
        world.goalNode = n;
        n.nodeType = NodeType.GOAL;
      }
    }
    isWorldEdited = true;
  }
  private int heuristicAt(PVector position) {
    return (int)(Math.abs(position.x - goalNode.position.x) + Math.abs(position.y - goalNode.position.y));
  }

  void solveStart() {
    startNode.score = 0;
    startNode.fs = startNode.score + heuristicAt(startNode.position);
    openNodes.add(startNode);
    solveNext();
  }

  void solveNext() {
    int minScore = Integer.MAX_VALUE;
    int minIndex = -1;
    Node u = null;
    for (int i = 0; i < openNodes.size (); i++) {
      Node targetNode = openNodes.get(i);
      if (targetNode.done) continue;
      if (targetNode.fs < minScore) {
        minScore = targetNode.fs;
        minIndex = i;
        u = targetNode;
      }
    }

    if (u == null) {
      return;
    }

    openNodes.remove(minIndex);
    u.done = true;

    if (u.nodeType == NodeType.GOAL) {
      return;
    }

    for (int i = 0; i < dx.length; i++) {
      if (u.position.y + dy[i] < 0 || u.position.y + dy[i] >= worldHeight|| u.position.x + dx[i] < 0 || u.position.x + dx[i] >= worldWidth) {
        continue;
      }
      Node v = getNode((int)u.position.x + dx[i], (int)u.position.y + dy[i]);
      if (v.done || v.nodeType == NodeType.WALL || v.nodeType == NodeType.OUTER) continue;
      //following number 1 is moving cost between node to node.
      if (u.score + 1 + heuristicAt(v.position) < v.fs) {
        v.score = u.score + 1;
        v.fs = v.score + heuristicAt(v.position);
        v.prev = u;

        if (openNodes.indexOf(v) == -1)openNodes.add(v);
      }
    }
    solveNext();
  }
}

